﻿
using System;
using System.Collections.Generic;
using PSW.CMS.Automate.Entity;
using Sdl.Core.Globalization;
using Sdl.Core.Settings;
using Sdl.LanguagePlatform.TranslationMemoryApi;
using Sdl.ProjectAutomation.Core;
using Sdl.ProjectAutomation.FileBased;
using Sdl.ProjectAutomation.Settings;
using System.Globalization;
using System.IO;
using Sdl.LanguagePlatform.TranslationMemory;
using Sdl.LanguagePlatform.Core.Tokenization;
using PSW.CMS.Common;


namespace PSW.CMS.Automate
{
    public class TradosProject
    {
        private TranslationRequestInfo translationRequestInfo;
        public FileBasedProject newProject{ get; set; }

        #region Constructor
        public TradosProject(TranslationRequestInfo translationRequestInfo)
        {
            translationRequestInfo.CheckParameter("translationRequestInfo");
            this.translationRequestInfo = translationRequestInfo;

            string templateFile = @"C:/TradosTest/sample/TradosTestTemplate.sdltpl";
            ProjectTemplateReference template = new ProjectTemplateReference(templateFile);

            newProject = new FileBasedProject(this.GetInfoForTemplateProject(), template);
            newProject.Save();

        }

        #endregion // Constructor

        #region Method

        public void PerfectMatch()
        {
            string mainPath = PathExtensions.GetWorkTargetDirPath(translationRequestInfo);
            this.AddFiles(newProject, mainPath, true);

            string previousFile = PathExtensions.GetWorkTempDirPath(translationRequestInfo, AppConfig.GetInstance().GetWorkAllBilingualFiles());
            string srcLocale = AppConfig.GetInstance().GetSourceLanguage();
            string trgLocale = translationRequestInfo.CultureName;

            if (translationRequestInfo.IsEnableBilingualFile)
            {
                string tmName = string.Format(@"C:\TradosTest\sample\test_{0}.sdltm", translationRequestInfo.GetRequestFolderName());
                string AllBilingualFile = PathExtensions.GetWorkTempDirPath(translationRequestInfo, AppConfig.GetInstance().GetWorkAllBilingualFiles());
                
                TMCreator tmc = new TMCreator();
                FileBasedTranslationMemory fileTm = tmc.CreateFileBasedTM(tmName, srcLocale, trgLocale);

                TMImporter tmi = new TMImporter();
                tmi.ImportFile(fileTm, AllBilingualFile);

                this.AddTm(newProject, tmName);
            }
            else
            {
                foreach (var tmName in translationRequestInfo.TranslationMemoryName)
                {
                    var tmFile = Path.Combine(translationRequestInfo.GetManualConfigurationInfo().TranslationMemoryFolderPath, tmName);
                    this.AddTm(newProject, tmFile);
                }
            }

            this.GetPerfectMatchTaskSettings(newProject);
            this.GetAnalyzeSettings(newProject, trgLocale);

            this.RunPrepare(newProject, trgLocale, previousFile);

            this.CreateReports(newProject, Path.Combine(Path.GetDirectoryName(newProject.FilePath), "Analysis.csv"));
            
            newProject.UpdateProject(newProject.GetProjectInfo());
            newProject.Save();
        }

        public void GenerateTranslatedFile()
        {
            string mainPath = PathExtensions.GetWorkRequestDirPath(translationRequestInfo, AppConfig.GetInstance().GetWorkProject(), translationRequestInfo.GetManualConfigurationInfo().Lang);
            var projectFiles = this.AddFiles(newProject, mainPath, true);

            ConvertFiles(newProject);

            this.GetExportTaskSettings(newProject);

            var files = newProject.GetTargetLanguageFiles();

            //AutomaticTask convertTask = newProject.RunAutomaticTask(newProject.GetTargetLanguageFiles().GetIds(), AutomaticTaskTemplateIds.GenerateTargetTranslations);
            AutomaticTask exportTask = newProject.RunAutomaticTask(files.GetIds(), AutomaticTaskTemplateIds.ExportFiles);

            newProject.UpdateProject(newProject.GetProjectInfo());
            newProject.Save();
        }

        #endregion // Method

        #region private Method

        public ProjectInfo GetInfoForTemplateProject()
        {
            ProjectInfo info = new ProjectInfo();

            info.Name = AppConfig.GetInstance().GetWorkProject() + "_" + DateTimeExtensions.GetLogicDatetime();

            string localProjectFolder = string.Format(
                CultureInfo.CurrentCulture,
                "{0}{1}{2}",
                PathExtensions.GetWorkRequestDirPath(translationRequestInfo),
                Path.AltDirectorySeparatorChar,
                info.Name);
            info.LocalProjectFolder = localProjectFolder;

            return info;
        }

        private ProjectFile[] AddFiles(FileBasedProject project, string folder, bool recursion)
        {
            project.AddFolderWithFiles(folder, recursion);
            ProjectFile[] projectFiles = project.GetSourceLanguageFiles();
            AutomaticTask scan = project.RunAutomaticTask(projectFiles.GetIds(), AutomaticTaskTemplateIds.Scan);
            return projectFiles;
        }
        
        private void AddTm(FileBasedProject project, string tmFilePath)
        {
            TranslationProviderConfiguration config = project.GetTranslationProviderConfiguration();
            TranslationProviderCascadeEntry tm = new TranslationProviderCascadeEntry(
                tmFilePath,
                false,
                true,
                false);
            config.Entries.Add(tm);

            project.UpdateTranslationProviderConfiguration(config);
        }

        private void ConvertFiles(FileBasedProject project)
        {
            ProjectFile[] files = project.GetSourceLanguageFiles();

            for (int i = 0; i < project.GetSourceLanguageFiles().Length; i++)
            {
                if (files[i].Role == FileRole.Translatable)
                {
                    Guid[] currentFileId = { files[i].Id };
                    AutomaticTask convertTask = project.RunAutomaticTask(currentFileId,　AutomaticTaskTemplateIds.ConvertToTranslatableFormat);
                    AutomaticTask copyTask = project.RunAutomaticTask(currentFileId, AutomaticTaskTemplateIds.CopyToTargetLanguages);
                }
            }
        }

        private void RunPrepare(FileBasedProject project, string trgLocale, string previousFolder)
        {
            ProjectInfo info = project.GetProjectInfo();
            string previousProjectPath = previousFolder;

            ConvertFiles(project);

            ProjectFile[] targetFiles = project.GetTargetLanguageFiles(new Language(CultureInfo.GetCultureInfo(trgLocale)));
            project.AddBilingualReferenceFiles(GetBilingualFileMappings(new Language(CultureInfo.GetCultureInfo(trgLocale)), targetFiles, previousProjectPath));

            EventHandler<TaskStatusEventArgs> perfectEventArgs = new EventHandler<TaskStatusEventArgs>(this.perfectEventHandle);
            AutomaticTask perfectMatchTask = project.RunAutomaticTask(targetFiles.GetIds(), AutomaticTaskTemplateIds.PerfectMatch, perfectEventArgs, (sender, e) => { });
            AutomaticTask preTranslateTask = project.RunAutomaticTask(targetFiles.GetIds(), AutomaticTaskTemplateIds.PreTranslateFiles);
            AutomaticTask analyzeTask = project.RunAutomaticTask(targetFiles.GetIds(), AutomaticTaskTemplateIds.AnalyzeFiles);

            project.SaveTaskReportAs(analyzeTask.Reports[0].Id, info.LocalProjectFolder + "/atr.xml", ReportFormat.Xml);
        }


        private void perfectEventHandle(object sender, TaskStatusEventArgs e)
        {
            switch (e.Status)
            {
                case TaskStatus.Assigned:
                    break;
                case TaskStatus.Cancelled:
                    break;
                case TaskStatus.Cancelling:
                    break;
                case TaskStatus.Completed:
                    break;
                case TaskStatus.Created:
                    break;
                case TaskStatus.Invalid:
                    break;
                case TaskStatus.Rejected:
                    break;
                case TaskStatus.Started:
                    break;
                case TaskStatus.Failed:
                    break;
                default:
                    break;
            }

        }

        private void GetAnalyzeSettings(FileBasedProject project,　string trgLocale)
        {
            Language trgLanguage = new Language(CultureInfo.GetCultureInfo(trgLocale));

            ISettingsBundle settings = project.GetSettings(trgLanguage);
            AnalysisTaskSettings analyzeSettings = settings.GetSettingsGroup<AnalysisTaskSettings>();
            TranslateTaskSettings pretranslateSettings = settings.GetSettingsGroup<TranslateTaskSettings>();

            analyzeSettings.ReportCrossFileRepetitions.Value = true;
            analyzeSettings.ReportInternalFuzzyMatchLeverage.Value = false;

            pretranslateSettings.ConfirmAfterApplyingExactMatch.Value = true;
            pretranslateSettings.LockExactMatchSegments.Value = false;
            pretranslateSettings.ConfirmAfterApplyingInContextExactMatch.Value = true;
            pretranslateSettings.LockContextMatchSegments.Value = true;

            project.UpdateSettings(trgLanguage, settings);
        }

        private void GetPerfectMatchTaskSettings(FileBasedProject project)
        {
            ISettingsBundle settings = project.GetSettings();
            PerfectMatchTaskSettings perfectMatchSettings = settings.GetSettingsGroup<PerfectMatchTaskSettings>();
            perfectMatchSettings.MarkAsPerfectMatchAndLock.Value = true;

            project.UpdateSettings(settings);
        }

        public void GetExportTaskSettings(FileBasedProject project)
        {
            ISettingsBundle settings = project.GetSettings();
            ExportFilesSettings exportTaskSettings = settings.GetSettingsGroup<ExportFilesSettings>();

            exportTaskSettings.ExportLocation.Value = PathExtensions.GetWorkRequestDirPath(translationRequestInfo, AppConfig.GetInstance().GetWorkProject(), "exportFolder");
            exportTaskSettings.ExportFileVersion.Value = ExportFileVersion.Native;

            project.UpdateSettings(settings);
        }

        private void CreateReports(FileBasedProject project, string path)
        {
            ProjectStatistics projectStats = project.GetProjectStatistics();
            TargetLanguageStatistics[] targetStats = projectStats.TargetLanguageStatistics;
            AnalysisStatistics analysisStats = targetStats[0].AnalysisStatistics;

            ProjectFile[] targetFiles = project.GetTargetLanguageFiles();
            FileInfo fileInfo = new FileInfo(path);
            using (StreamWriter streamWriter = fileInfo.CreateText())
            {
                streamWriter.WriteLine(",Repetitions,,,,PerfectMatch,,,,Exact,,,,Total");
                streamWriter.WriteLine("File,Segments,Characters,Placeables,Percent,Segments,Characters,Placeables,Percent,Segments,Characters,Placeables,Percent,Segments,Characters,Placeables");
                foreach (var tfile in targetFiles)
                {
                    if (null != tfile.AnalysisStatistics)
                    {
                        var total = (tfile.AnalysisStatistics.Total.Characters == 0) ? 1 : tfile.AnalysisStatistics.Total.Characters;
                        string line = "";
                        line += tfile.LocalFilePath.ToString() + ",";
                        line += tfile.AnalysisStatistics.Repetitions.Segments.ToString() + ",";
                        line += tfile.AnalysisStatistics.Repetitions.Characters.ToString() + ",";
                        line += tfile.AnalysisStatistics.Repetitions.Placeables.ToString() + ",";
                        line += (tfile.AnalysisStatistics.Repetitions.Characters / total * 100).ToString() + ",";
                        line += tfile.AnalysisStatistics.Perfect.Segments.ToString() + ",";
                        line += tfile.AnalysisStatistics.Perfect.Characters.ToString() + ",";
                        line += tfile.AnalysisStatistics.Perfect.Placeables.ToString() + ",";
                        line += (tfile.AnalysisStatistics.Perfect.Characters / total * 100).ToString() + ",";
                        line += tfile.AnalysisStatistics.Exact.Segments.ToString() + ",";
                        line += tfile.AnalysisStatistics.Exact.Characters.ToString() + ",";
                        line += tfile.AnalysisStatistics.Exact.Placeables.ToString() + ",";
                        line += (tfile.AnalysisStatistics.Exact.Characters / total * 100).ToString() + ",";
                        line += tfile.AnalysisStatistics.Total.Segments.ToString() + ",";
                        line += tfile.AnalysisStatistics.Total.Characters.ToString() + ",";
                        line += tfile.AnalysisStatistics.Total.Placeables.ToString() + ",";
                        streamWriter.WriteLine(line);
                    }
                }
                streamWriter.Close();

            }
            string result = "Analysis result for all files:\n\n";

            result += "\nPerfect matches\n";
            result += "Segments: " + analysisStats.Perfect.Segments.ToString() + "\n";
            result += "Words: " + analysisStats.Perfect.Words.ToString() + "\n";
            result += "Characters: " + analysisStats.Perfect.Characters.ToString() + "\n";
            result += "Tags: " + analysisStats.Perfect.Tags.ToString() + "\n";
            result += "Placeables: " + analysisStats.Perfect.Placeables.ToString() + "\n";

            result += "\nContext matches\n";
            result += "Segments: " + analysisStats.InContextExact.Segments.ToString() + "\n";
            result += "Words: " + analysisStats.InContextExact.Words.ToString() + "\n";
            result += "Characters: " + analysisStats.InContextExact.Characters.ToString() + "\n";
            result += "Tags: " + analysisStats.InContextExact.Tags.ToString() + "\n";
            result += "Placeables: " + analysisStats.InContextExact.Placeables.ToString() + "\n";


            result += "\nExact matches\n";
            result += "Segments: " + analysisStats.Exact.Segments.ToString() + "\n";
            result += "Words: " + analysisStats.Exact.Words.ToString() + "\n";
            result += "Characters: " + analysisStats.Exact.Characters.ToString() + "\n";
            result += "Tags: " + analysisStats.Exact.Tags.ToString() + "\n";
            result += "Placeables: " + analysisStats.Exact.Placeables.ToString() + "\n";

            for (int i = 0; i < analysisStats.Fuzzy.Length; i++)
            {
                string rangeMax = analysisStats.Fuzzy[i].Band.MaximumMatchValue.ToString();
                string rangeMin = analysisStats.Fuzzy[i].Band.MinimumMatchValue.ToString();

                result += "\nFuzzy matches " + rangeMax + "% to " + rangeMin + "%\n";
                result += "Segments: " + analysisStats.Fuzzy[i].Segments.ToString() + "\n";
                result += "Words: " + analysisStats.Fuzzy[i].Words.ToString() + "\n";
                result += "Characters: " + analysisStats.Fuzzy[i].Characters.ToString() + "\n";
                result += "Tags: " + analysisStats.Fuzzy[i].Tags.ToString() + "\n";
                result += "Placeables: " + analysisStats.Fuzzy[i].Placeables.ToString() + "\n";
            }

            result += "\nRepetitions\n";
            result += "Segments: " + analysisStats.Repetitions.Segments.ToString() + "\n";
            result += "Words: " + analysisStats.Repetitions.Words.ToString() + "\n";
            result += "Characters: " + analysisStats.Repetitions.Characters.ToString() + "\n";
            result += "Tags: " + analysisStats.Repetitions.Tags.ToString() + "\n";
            result += "Placeables: " + analysisStats.Repetitions.Placeables.ToString() + "\n";

            result += "\nNew segments\n";
            result += "Segments: " + analysisStats.New.Segments.ToString() + "\n";
            result += "Words: " + analysisStats.New.Words.ToString() + "\n";
            result += "Characters: " + analysisStats.New.Characters.ToString() + "\n";
            result += "Tags: " + analysisStats.New.Tags.ToString() + "\n";
            result += "Placeables: " + analysisStats.New.Placeables.ToString() + "\n";

            result += "\nTotal\n";
            result += "Segments: " + analysisStats.Total.Segments.ToString() + "\n";
            result += "Words: " + analysisStats.Total.Words.ToString() + "\n";
            result += "Characters: " + analysisStats.Total.Characters.ToString() + "\n";
            result += "Tags: " + analysisStats.Total.Tags.ToString() + "\n";
            result += "Placeables: " + analysisStats.Total.Placeables.ToString() + "\n";

            Console.Write(result);
        }

        public BilingualFileMapping[] GetBilingualFileMappings(Language targetLanguage, ProjectFile[] translatableFiles, string previousProjectPath)
        {
            List<BilingualFileMapping> mappings = new List<BilingualFileMapping>();
            List<FileInfo> lFi = new List<FileInfo>();
            searchFile(previousProjectPath, lFi);
            foreach (ProjectFile file in translatableFiles)
            {
                string fileName = file.Name.Split('.')[0];
                foreach (var item in lFi)
                {
                    string itemName = item.Name.Split('.')[0];
                    if (fileName.Equals(itemName))
                    {
                        BilingualFileMapping mapping = new BilingualFileMapping()
                        {
                            BilingualFilePath = item.FullName,
                            Language = targetLanguage,
                            FileId = file.Id
                        };
                        mappings.Add(mapping);
                        break;
                    }
                }
            }
            return mappings.ToArray();
        }

        internal static void searchFile(string dir, List<FileInfo> fi)
        {
            string[] files = Directory.GetFiles(dir);
            foreach (string s in files)
            {
                if (s.EndsWith(".sdlxliff"))
                {
                    fi.Add(new FileInfo(Path.Combine(dir, s)));
                }
                //else if (s.EndsWith(".ttx"))
                //{
                //    fi.Add(new FileInfo(Path.Combine(dir, s)));
                //}
            }
            string[] dirs = Directory.GetDirectories(dir);
            foreach (string s in dirs)
            {
                searchFile(s, fi);
            }
        }
        #endregion // private Method
    }

    public class TMCreator
    {
        public FileBasedTranslationMemory CreateFileBasedTM(string tmPath, string sourceLng, string targetLng)
        {
            FileBasedTranslationMemory tm = new FileBasedTranslationMemory(
                tmPath,
                "Description: This is a sample TM",
                CultureInfo.GetCultureInfo(sourceLng),
                CultureInfo.GetCultureInfo(targetLng),
                this.GetFuzzyIndexes(),
                this.GetRecognizers());

            tm.LanguageResourceBundles.Clear();

            tm.Save();
            return tm;
        }

        private FuzzyIndexes GetFuzzyIndexes()
        {
            return FuzzyIndexes.SourceCharacterBased |
                FuzzyIndexes.SourceWordBased |
                FuzzyIndexes.TargetCharacterBased |
                FuzzyIndexes.TargetWordBased;
        }

        private BuiltinRecognizers GetRecognizers()
        {
            return BuiltinRecognizers.RecognizeAcronyms |
                BuiltinRecognizers.RecognizeDates |
                BuiltinRecognizers.RecognizeNumbers |
                BuiltinRecognizers.RecognizeTimes |
                BuiltinRecognizers.RecognizeVariables |
                BuiltinRecognizers.RecognizeMeasurements;
        }
    }

    public class TMImporter
    {
        public void ImportFile(FileBasedTranslationMemory tm, string importFilePath)
        {
            //FileBasedTranslationMemory tm = new FileBasedTranslationMemory(tmPath);
            TranslationMemoryImporter importer = new TranslationMemoryImporter(tm.LanguageDirection);

            importer.ChunkSize = 20;
            this.GetImportSettings(importer.ImportSettings);
            importer.BatchImported += new EventHandler<BatchImportedEventArgs>(this.importer_BatchImported);
            if (File.Exists(importFilePath))
            {
                importer.Import(importFilePath);
            }
            else if (Directory.Exists(importFilePath))
            {
                var fi = new List<FileInfo>();
                TradosProject.searchFile(importFilePath, fi);
                foreach (var file in fi)
                {
                    importer.Import(file.FullName);
                }
            }
            else
            {
                throw new FileNotFoundException("Directory Not Found", importFilePath);
            }
        }

        private void importer_BatchImported(object sender, BatchImportedEventArgs e)
        {
            string info;
            ImportStatistics stats = e.Statistics;

            info = "Total read: " + stats.TotalRead + "\n";
            info += "Total imported: " + stats.TotalImported + "\n";
            info += "TUs added: " + stats.AddedTranslationUnits + "\n";
            info += "TUs discarded: " + stats.DiscardedTranslationUnits + "\n";
            info += "TUs merged: " + stats.MergedTranslationUnits + "\n";
            info += "Errors: " + stats.Errors + "\n";

            e.Cancel = false;
            if (0 != stats.Errors)
            {
                //throw new Sdl.LanguagePlatform.Core.LanguagePlatformException(LanguagePlatform.Core.ErrorCode.TMImportFieldNotExists);
            }
        }

        private void GetImportSettings(ImportSettings importSettings)
        {
            importSettings.CheckMatchingSublanguages = false;

            importSettings.ExistingFieldsUpdateMode = ImportSettings.FieldUpdateMode.Merge;

            ConfirmationLevel[] levels = { ConfirmationLevel.ApprovedTranslation, ConfirmationLevel.Translated };
            importSettings.ConfirmationLevels = levels;

            importSettings.InvalidTranslationUnitsExportPath = @"c:\temp\invalid.tmx";

            importSettings.OverwriteExistingTUs = true;

            importSettings.PlainText = false;
            importSettings.TagCountLimit = 10;

            importSettings.IncrementUsageCount = true;
        }
    }
}
